/* GAMEPACK 01
This software is released "as is" and no warranty is given. Try and use at your own responsability.
The Gamepack 01 is released under the Creative Commons License 3.0 Attribution / Share-alike / No-commercial.
You are free to modify the program and redistribute it but need to:
- released your derivative works with the same license;
- make credits to me, the original creator (Leonardo Miliani - www.leonardomiliani.com)
- ask no money for it
*/

#include "nokia_3310_lcd.h"

//************************************************************************
// IMPORTANT SYSTEM VARIABLES - DON'T ALTER THEM  (FROM HERE....)
//************************************************************************
//keypad debounce parameter
#define DEBOUNCE_MAX 15
#define DEBOUNCE_ON  10
#define DEBOUNCE_OFF 3 

#define NUM_KEYS 5

// joystick number
#define UP_KEY 3
#define LEFT_KEY 0
#define CENTER_KEY 1
#define DOWN_KEY 2
#define RIGHT_KEY 4

// adc preset value, represent top value,incl. noise & margin,that the adc reads, when a key is pressed
// set noise & margin = 30 (0.15V@5V)
int adc_key_val[5] ={30, 150, 360, 535, 760 };

// debounce counters
byte button_count[NUM_KEYS];
// button status - pressed/released
byte button_status[NUM_KEYS];
// button on flags for user program 
byte button_flag[NUM_KEYS];

Nokia_3310_lcd lcd=Nokia_3310_lcd();

//************************************************************************
// (.... TO HERE)
//************************************************************************


//************************************************************************
// START GAMES GLOBAL VARIABLES 
//************************************************************************
#define WIDTH 14 // screen width in chars
#define HEIGHT 6 //screen height in chars
#define SNAKE_BODY 111 //char for snake body
#define BLANK 32 //blank char
#define SNAKE_FRUIT 124 //fruit for Snake game
#define INITIAL_SNAKE_SPEED 500 //initial delay in ms between 2 moves of the snake
#define MAX_SNAKE_LENGHT 40 //max lenght of the snake in segments
#define MAX_SNAKE_SPEED 200 //min delay in ms between 2 moves of the snake
#define INC_SNAKE_SPEED 40 //delay decrement of snake move delay
#define INITIAL_SNAKE_LENGHT 4 //initial lenght of snake
#define PLAYER 125 //player char for Eat'n'Go game
#define PILL 124 //pill char for Eat'n'Go game
#define ENEMY 123 //enemy char for Eat'n'Go game
#define PLAYER_SHIP 126 //player's space ship for Space Ace game
#define ALIEN_SHIP_LX 127 //alien ship - left piece
#define ALIEN_SHIP_DX 128 //alien ship - right piece
#define EXPLOSION 42 //explosion char
#define MAX_PLAYER_SHOTS 1 //only 1 shot at a time
#define SHOT_SPEED 200
#define PLAYER_SHOT 45 //player shot char
#define MAX_ALIEN_SPEED 200 
#define START_ALIEN_SPEED 500

int snake_pos_fruit[2]; //position of next fruit
char snake[MAX_SNAKE_LENGHT][2]; //position of the segments of the snake (snake[.][0]=x snake[.][1]=y)
int snake_speed; //snake speed
int snake_lenght; //snake lenght
int snake_head; //index for the snake head
int snake_tail; //index for the snake tail
int snake_vert_dir; //snake vertical direction (-1/+1)
int snake_hor_dir; //snake horizontal direction (-1/+1)
byte pills=0; //pills into the screen for Eat'n'Go game
unsigned long time_next_move; // var used in both games - keeps the time for next move of something
int level; //game level
int score; //game score
char screen[WIDTH][HEIGHT]; //this array keeps a mirror of the screen image (the chars that will be designed)
byte exit_game; //used in several points of the code to manage the exiting from loops

int alien_shot_x; // alien shot coords
int alien_shot_y; 
int shot_x; //player shot coords
int shot_y;
byte player_shots; 
byte num_aliens;
byte alien_shots;
int player_x; //player coords
int player_y;
int alien_x;
int alien_y;
unsigned long aliens_speed;


//************************************************
// GAME MENUS
//************************************************
#define INGAME_NUM_MENU_ITEM 2
#define INGAME_MENU_X 10
#define INGAME_MENU_Y 2

#define MAIN_NUM_MENU_ITEM 3
#define MAIN_MENU_X 10
#define MAIN_MENU_Y 2

char current_menu_item;

//elements of main menu
char main_menu_items[MAIN_NUM_MENU_ITEM][10]={
        "Eat'n'Go",
        "Snake",
        "Space Ace"
};

//elements of pause menu
char ingame_menu_items[INGAME_NUM_MENU_ITEM][7]={
	"Resume",
	"Exit"
};

//main menu setup
void main_MENU(void){
    byte i;
    
        lcd.LCD_3310_clear();
        lcd.LCD_3310_write_string(9,0,"GAMEPACK 01", MENU_NORMAL);
        lcd.LCD_3310_write_string(MAIN_MENU_X, MAIN_MENU_Y, main_menu_items[0], MENU_HIGHLIGHT );
        for (i=1; i<MAIN_NUM_MENU_ITEM; i++){
            lcd.LCD_3310_write_string(MAIN_MENU_X, MAIN_MENU_Y+i, main_menu_items[i], MENU_NORMAL);
        }
        for(i=0;i<NUM_KEYS;i++) {
            button_flag[i]=0;
        }
        current_menu_item=0;
}

//pause menu setup
void ingame_MENU(void){
    byte i;
        lcd.LCD_3310_write_string(INGAME_MENU_X, INGAME_MENU_Y, ingame_menu_items[0], MENU_HIGHLIGHT );
        for (i=1; i<INGAME_NUM_MENU_ITEM; i++){
            lcd.LCD_3310_write_string(INGAME_MENU_X, INGAME_MENU_Y+i, ingame_menu_items[i], MENU_NORMAL);
        }	
        current_menu_item=0;
}

// IMPORTANT - This code is needed to manage the joystick - don't change it
void setup() {
    byte i,j,k; 
    // setup interrupt-driven keypad arrays  
    // reset button arrays
    for(byte i=0; i<NUM_KEYS; i++){
        button_count[i]=0;
        button_status[i]=0;
        button_flag[i]=0;
    }

    // Setup timer2 -- Prescaler/256
    TCCR2A &= ~((1<<WGM21) | (1<<WGM20));
    TCCR2B &= ~(1<<WGM22);
    TCCR2B = (1<<CS22)|(1<<CS21);      
    ASSR |=(0<<AS2);

    // Use normal mode  
    TCCR2A =0;    
    //Timer2 Overflow Interrupt Enable  
    TIMSK2 |= (0<<OCIE2A);
    TCNT2=0x6;  // counting starts from 6;  
    TIMSK2 = (1<<TOIE2);                    
    SREG|=1<<SREG_I;
    lcd.LCD_3310_init(); //initialize the LCD display
    randomSeed(millis()); //initialize the random generator
}


//main Arduino function - used to call the game chooser
void loop() {
    choose_games();
}


//From here the user chooses the game to play
void choose_games() {
int i,done;

    main_MENU();
    time_next_move=millis();
    done=0;
    do
    {
        //light off the display after 10 secs of user inactivity
        if (millis()-time_next_move>10000 && done==0) {
            lcd.LCD_3310_light(LIGHT_OFF);
            done=1;
        }
        for(i=0; i<NUM_KEYS; i++){
            if(button_flag[i] !=0){
                time_next_move=millis();
                //light on the display when the user push a button
                lcd.LCD_3310_light(LIGHT_ON);
                done=0;
                button_flag[i]=0;  // reset button flag
                    switch(i){
                        case UP_KEY:
                            // current item to normal display
                            lcd.LCD_3310_write_string(MAIN_MENU_X, MAIN_MENU_Y + current_menu_item, main_menu_items[current_menu_item], MENU_NORMAL );
                            current_menu_item -=1;
                            if(current_menu_item <0)  current_menu_item = MAIN_NUM_MENU_ITEM -1;
                            // next item to highlight display
    			    lcd.LCD_3310_write_string(MAIN_MENU_X, MAIN_MENU_Y + current_menu_item, main_menu_items[current_menu_item], MENU_HIGHLIGHT );
			    break;
                        case DOWN_KEY:
                            // current item to normal display
                            lcd.LCD_3310_write_string(MAIN_MENU_X, MAIN_MENU_Y + current_menu_item, main_menu_items[current_menu_item], MENU_NORMAL );
			    current_menu_item +=1;
			    if(current_menu_item >(MAIN_NUM_MENU_ITEM-1))  current_menu_item = 0;
			    // next item to highlight display
			    lcd.LCD_3310_write_string(MAIN_MENU_X, MAIN_MENU_Y + current_menu_item, main_menu_items[current_menu_item], MENU_HIGHLIGHT );
			    break;
                        case CENTER_KEY:
                            //start Eat'n'Go
                            if (current_menu_item==0) { 
                                eatngo_game();
                                main_MENU();
                                time_next_move=millis();
                                break;
                            }
                            //start Snake
                            if (current_menu_item==1) { 
                                snake_game();
                                main_MENU();
                                time_next_move=millis();
                                break;
                            }
                            //start Space-Ace
                            if (current_menu_item==2) {
                                spaceace_game();
                                main_MENU();
                                time_next_move=millis();
                                break;
                            }
			    break;   
                    }
        }
    }
    } while (1); //infinite loop
}

//Start Snake game
void snake_game() {
    char* game_hints[]= {"* S N A K E *", "Eat the fruits", "Use joystick", "to move"};
    byte pos_hints[]= {3, 0, 6, 21};
    byte num_hints=4;
    byte x,y;
    byte x1,y1;
    byte i,j,k;
    byte movement, death;
    int x1m, y1m, x2m, y2m;
    unsigned long prev_time, time;
    
    i=hints(game_hints, pos_hints, num_hints);
    if (i==2) { return; }
    do {
        initialize_snake();
        for(i=0; i<NUM_KEYS; i++) { // reset buttons
            button_flag[i]=0;  
        }
        k=0;
        draw_display();
        time=millis();
        time_next_move=time+snake_speed;
        exit_game=0;
        death=0;
        
        // start the game
        do {
            // check if the player has moved the joystick
            while (millis() < time_next_move) {
                delay(5);
                for(i=0; i<NUM_KEYS; i++) {
                    if (button_flag[i] !=0) {
                        button_flag[i]=0;
                        switch(i){
                            case UP_KEY: 
                                snake_vert_dir=-1;
                                snake_hor_dir=0;
                                break;
                            case DOWN_KEY:
                                snake_vert_dir=1;
                                snake_hor_dir=0;
                                break;
                            case LEFT_KEY:
                                snake_vert_dir=0;
                                snake_hor_dir=-1;
                                break;
                            case RIGHT_KEY:
                                snake_vert_dir=0;
                                snake_hor_dir=1;
                                break;
                            case CENTER_KEY:
                                game_pause();     
                                if (exit_game!=0) {
                                    level=0;
                                    break;
                                }
                                break;
                        }
                    }
                }
            }
            // player want to terminate the game
            if (level==0) {
                exit_game=1; 
                break;
            }
            // there's a move
           time_next_move=millis()+snake_speed;
           death=0;
           death=move_snake();
        } while (exit_game==0 && death==0);
        lcd.LCD_3310_clear();
        if (death==1 || level==0) {
            if (exit_game == 1) { score=-1; }
            game_over(score);
            level=0;
        }
    } while (level>0);
}


//Game over routine
void game_over(int gamescore) {
    lcd.LCD_3310_write_string(15,0,"GAME OVER", MENU_NORMAL);
    if (gamescore>=0) { show_score(2, score); }
    lcd.LCD_3310_write_string(36,5,"OK",MENU_HIGHLIGHT);
    waitfor_OKkey();
}


//initialize Snake
void initialize_snake() {
char x,y;
int i,j;

    //setup initial variables
    score=0;
    level=1;
    // clear the screen array
    for (x=0;x<WIDTH;x++) {
        for (y=0;y<HEIGHT;y++) {
            screen[x][y]=BLANK;
        }
    }
    // setup of the snake
    snake_vert_dir=0;
    snake_hor_dir=1; //initial direction is RIGHT
    snake_speed=INITIAL_SNAKE_SPEED;
    snake_tail=0;
    snake_lenght=INITIAL_SNAKE_LENGHT;
    snake_head=snake_tail+snake_lenght-1;
    //fill the array of the segments of the snake with initial positions
    for (i=0;i<MAX_SNAKE_LENGHT;i++) {
        if (i<snake_lenght) {
            snake[i][0]=0+i;
            snake[i][1]=2;
        } else { 
            snake[i][0]=0;
            snake[i][1]=0;
        }
    }
    //insert the snake in the screen
    for (i=0;i<snake_lenght;i++) {
        screen[snake[i][0]][snake[i][1]]=SNAKE_BODY;
    }
    draw_fruit(); //put 1 fruit
}


//draw the fruit into the screen
void draw_fruit() {
int i,x,y;
    // check if the position of the upcoming fruit will be over a segment of the snake
    i=0;
    do {
        x=random(0,WIDTH);
        y=random(0,HEIGHT);
        if (screen[x][y]==BLANK) { i=1; }
    } while (i==0);
    //insert the fruit into the screen
    screen[x][y]=SNAKE_FRUIT;
}


//move the snake - returns '1' if the snake has died
char move_snake() {
int grow=0;
int new_head=0;
int x,y;

    new_head=snake_head+1;
    if (new_head==MAX_SNAKE_LENGHT) {
        new_head=0;
    }
    // calculate the next position of the snake head
    snake[new_head][0]=snake[snake_head][0]+snake_hor_dir;
    snake[new_head][1]=snake[snake_head][1]+snake_vert_dir;
    snake_head=new_head;
    // check if the snake has gone out of the screen field
    if (snake[snake_head][0]<0 || snake[snake_head][0]>=WIDTH ) { return 1; }
    if (snake[snake_head][1]<0 || snake[snake_head][1]>=HEIGHT ) { return 1; }

    // check if the snake has eaten a fruit
    x=screen[snake[snake_head][0]][snake[snake_head][1]];
    screen[snake[snake_head][0]][snake[snake_head][1]]=SNAKE_BODY;
    if (x==SNAKE_FRUIT) {
        draw_fruit(); //new fruit
        update_score(1); //update score
        //grows up the snake
        if (snake_lenght<MAX_SNAKE_LENGHT-2) {
            snake_lenght++;
            grow=1;
        }
    // the snake has eaten itself!
    } else if (x==SNAKE_BODY) {
        return 1;
    } else {
        grow=0;
    }
    //insert the new head into the screen

    //check if the snake doesn't have to grow up: if true, deletes the last segment (tail)
    if (grow==0) {
        screen[snake[snake_tail][0]][snake[snake_tail][1]]=BLANK;
        if (++snake_tail==MAX_SNAKE_LENGHT) { snake_tail=0; }
    }
    //update the screen
    draw_display();
    return 0;
}


//update the score
void update_score(int addscore) {
    score +=addscore;
    if ((score%4)==0) { level_up(); } //next level?
}


//update level
void level_up() {
    level++;
    //check if snake speed must be increased
    if (snake_speed>MAX_SNAKE_SPEED) {
        snake_speed-=INC_SNAKE_SPEED;
    }
}


//show the final score
void show_score(int y, int result) {
int b=0;
int c=0;
int t=100;
byte i=0;
byte k=0;

    /* I could use the "sprintf" function but it increased
    the executable lenght of about 1K!!**/
    
    lcd.LCD_3310_write_string(12,y,"Score: ",MENU_NORMAL);
    if (result>999) { result=999; }
    k=4;
    do {
        c= result/t;
        if (c==0) {
        } else {
            b=result%10;
            printat(7+k,y,(char)b+48);
            i++;
            k--;
            b=b/10;
        }
        t/=10;
    } while (t>0 && k>0); 
}


/* Eat'n'go */
void eatngo_game() {
    char* game_hints[]= {"* EAT'N'GO *", "Eat the pills,", "avoid enemy", "Use joystick", "to move"};
    byte pos_hints[]= {6, 0, 9, 6, 21};
    byte num_hints=5;
    int x,y;
    int x1,y1;
    int i,j,k;
    byte movement, death;
    int x1m, y1m, x2m, y2m;
    unsigned long prev_time;
    unsigned long time=millis();
    byte cell;
    
    i=hints(game_hints, pos_hints, num_hints);
    if (i==2) { return; }
    level=1;
    time_next_move=500;
    score=0;
    do {
        eatngo_fill_display();
        for(i=0; i<NUM_KEYS; i++) {
            button_flag[i]=0;  // reset button flag
        }
        //initial setup
        //enemy is randomly placed at start
        x1m=random(0,WIDTH); 
        y1m=random(0,HEIGHT); 
        cell=screen[x1m][y1m];
        screen[x1m][y1m]=ENEMY;
        x2m=x1m;
        y2m=y1m;     
        k=0;
        time_next_move -=(level-1)*20;
        do {
            x1=random(0,WIDTH); //player is randomly placed at start
            y1=random(0,HEIGHT); //avoiding to put him over the enemy cell
            if (screen[x1][y1]==PILL) {
                x=x1;
                y=y1;
                screen[x][y]=PLAYER;
                k=1;
            }
        } while (k==0);
        eatngo_display();
        exit_game=0;
        death=0;
        do {
            movement=0;
            for(i=0; i<NUM_KEYS; i++) {
                if (button_flag[i] !=0) {
                    button_flag[i]=0;  // reset button flag
                    switch(i){
                        case UP_KEY:
                            //up direction
                            if (y>0) {
                                y--;
                            }
                            movement=1;
                            break;
                        case DOWN_KEY:
                            //down direction
                            if (y<(HEIGHT-1)) {
                                y++;
                            }
                            movement=1;
                            break;
                        case LEFT_KEY:
                            //left direction
                            if (x>0) {
                                x--;
                            }
                            movement=1;
                            break;
                        case RIGHT_KEY:
                            //right direction
                            if (x<(WIDTH-1)) {
                                x++;
                            }
                            movement=1;
                            break;
                        case CENTER_KEY:
                            //central button
                            game_pause();    
                            if (exit_game==0) {
                                eatngo_display(); //game will continue
                            } else {
                                level=0; //game will end
                                break;
                            }
                            break;
                    }
                }
            }
            if (movement==1) {
                //the player has to be moved
                screen[x1][y1]=BLANK;
                if (screen[x][y]==ENEMY) {
                    death=1;
                    break;
                }
                if (screen[x][y]==PILL) { score++; }
                screen[x][y]=PLAYER;
                x1=x;
                y1=y;
                eatngo_display();
                movement=0;
            }
            //enemy movement
            time=millis();
                //calculate if the enemy has to be moved
            if (time-prev_time>time_next_move) {
                prev_time=time;
                screen[x2m][y2m]=cell;
                // the higher the level is the more "intelligents" the enemy moves are
                j=20-level;
                if (j<1) { j=1; }
                if (random(1,j)==1) { 
                    // player unlucky: the enemy will "target" him
                    if (x2m>x) { 
                        x1m=-1;
                    } else {
                        x1m=1;
                    }
                    if (y2m>y) {
                        y1m=-1;
                    } else {
                        y1m=1;
                    }
                    x2m+=x1m;
                    y2m+=y1m;
                } else {
                    // player is lucky: enemy random movement
                    do {
                        x1m=random(-1,2);
                    } while ((x2m+x1m)<0 || (x2m+x1m)>=WIDTH);
                    x2m+=x1m;
                    do {
                        y1m=random(-1,2);
                    } while ((y2m+y1m)<0 || (y2m+y1m)>=HEIGHT);
                    y2m+=y1m;
                }
                if (screen[x2m][y2m]==PLAYER) {
                    //enemy has caught the player
                    death=1;
                    break;
                }
                cell=screen[x2m][y2m];
                screen[x2m][y2m]=ENEMY;
                eatngo_display();
            }
        } while (pills>0 && exit_game==0 && death==0);
        lcd.LCD_3310_clear();
        if (death==1 || level==0) {
            //game over
            game_over(score);
            level=0;
        } else {
            //next level
            lcd.LCD_3310_write_string(12,1,"WELL DONE!", MENU_NORMAL);
            lcd.LCD_3310_write_string(9,2,"Prepare for",MENU_NORMAL);
            lcd.LCD_3310_write_string(12,3,"next level",MENU_NORMAL);
            lcd.LCD_3310_write_string(36,5,"OK",MENU_HIGHLIGHT);
            waitfor_OKkey();
            //score+=5;
            level++;
        }
    } while (level>0);
}


//Eat'n'Go initial display setup: the screen is full of pills
void eatngo_fill_display() {
    char xx,yy,k;
  
    lcd.LCD_3310_clear();
    for (xx=0; xx<WIDTH; xx++){
        for (yy=0; yy<HEIGHT; yy++){
            screen[xx][yy]=PILL;
        }
    }	
}


//draw the Eat'n'go game field into the screen
void eatngo_display() {
byte i,j,k;

    k=0;
    for (i=0;i<WIDTH;i++) {
        for (j=0;j<HEIGHT;j++) {
            if (screen[i][j]==PILL) {
                k++;
            }
            printat(i,j,screen[i][j]);
        }
    }
    pills=k;
}


// this menu will appear when the game is in pause (when the user presses the central button during a game)
void game_pause() {
byte i,escape;

    exit_game=0;
    lcd.LCD_3310_clear();
    lcd.LCD_3310_write_string(27,0,"Pause", MENU_NORMAL);
    ingame_MENU();
    escape=0;
    do
    {
        for(i=0; i<NUM_KEYS; i++){
        if(button_flag[i] !=0){
            button_flag[i]=0;  // reset button flag
                switch(i){
                    case UP_KEY:
                        // current item to normal display
                        lcd.LCD_3310_write_string(INGAME_MENU_X, INGAME_MENU_Y + current_menu_item, ingame_menu_items[current_menu_item], MENU_NORMAL );
			current_menu_item -=1;
			if(current_menu_item <0)  current_menu_item = INGAME_NUM_MENU_ITEM -1;
			// next item to highlight display
			lcd.LCD_3310_write_string(INGAME_MENU_X, INGAME_MENU_Y + current_menu_item, ingame_menu_items[current_menu_item], MENU_HIGHLIGHT );
			break;
                    case DOWN_KEY:
		        // current item to normal display
			lcd.LCD_3310_write_string(INGAME_MENU_X, INGAME_MENU_Y + current_menu_item, ingame_menu_items[current_menu_item], MENU_NORMAL );
			current_menu_item +=1;
			if(current_menu_item >(INGAME_NUM_MENU_ITEM-1))  current_menu_item = 0;
			// next item to highlight display
			lcd.LCD_3310_write_string(INGAME_MENU_X, INGAME_MENU_Y + current_menu_item, ingame_menu_items[current_menu_item], MENU_HIGHLIGHT );
			break;
                    case CENTER_KEY:
                        //the user has choosen to resume the game
                        if (current_menu_item==0) { 
                            lcd.LCD_3310_clear();
                            escape=1;
                            break;
                        }
                        //the user has choosen to exit the game
                        if (current_menu_item==1) { 
                            lcd.LCD_3310_clear();
                            exit_game=1; 
                            escape=1;
                            break;
                        }
			break;   
                    }
            }
        }
    } while (escape==0);
}


//draw the game field of Snake into the screen
void draw_display() {
byte x,y;

  lcd.LCD_3310_clear();
    for (x=0;x<WIDTH;x++) {
        for (y=0;y<HEIGHT;y++) {
            printat(x,y,screen[x][y]);
        }
    }
}


//*********************************
//       S P A C E - A C E
//       A shoot'em up game
//*********************************
void spaceace_game() {
    char* game_hints[]= {"* SPACE ACE *", "Shoot at the", "alien ships", "avoiding", "enemy missles"};
    byte pos_hints[]= {3, 9, 9, 18, 3};
    byte num_hints=5;
    
    int gamble;
    byte x,y;
    byte x1,y1;
    byte i,j,k;
    byte movement, death;
    int x1m, y1m, x2m, y2m;
    unsigned long prev_time;
    unsigned long time=millis();
    unsigned long shot_time;
    unsigned long alien_time;
    unsigned long alien_shot_time;
    unsigned long next_alien_shot;
    
    i=hints(game_hints, pos_hints, num_hints);
    if (i==2) { return; }
    level=1;
    aliens_speed=START_ALIEN_SPEED;
    score=0;
    player_shots=0;
    num_aliens=0;
    time_next_move=500;
    for(i=0; i<NUM_KEYS; i++) {
        button_flag[i]=0;  // reset button flag
    }
    //initial setup
    alien_time=millis()+aliens_speed;
    spaceace_init_display();
    draw_display();
    y=player_y;
    x=player_x;
    exit_game=0;
    death=0;
    alien_shots=0;
    next_alien_shot=millis()+5000;
    do {
        movement=0;
        for(i=0; i<NUM_KEYS; i++) {
            if (button_flag[i] !=0) {
                button_flag[i]=0;  // reset button flag
                switch(i){
                    case UP_KEY:
                        //up direction
                        if (y>0) {
                            y--;
                        }
                        movement=1;
                        break;
                    case DOWN_KEY:
                        //down direction
                        if (y<(HEIGHT-1)) {
                            y++;
                        }
                        movement=1;
                        break;
                    case RIGHT_KEY:
                        //fire!
                        if (player_shots<MAX_PLAYER_SHOTS) {
                            player_shots++;
                            shot_x=player_x;
                            shot_y=player_y;
                            shot_time=millis()+SHOT_SPEED;
                        }
                        movement=1;
                        break;
                    case CENTER_KEY:
                        //central button
                        game_pause();    
                        if (exit_game==0) {
                            draw_display(); //game will continue
                        } else {
                            level=0; //game will end
                            break;
                        }
                        break;
                }
            }
        }
        //player shot
        if (player_shots != 0 ) {
            if (millis()>shot_time) {
                move_player_shots();
                if (player_shots!=0) { 
                    shot_time=millis()+SHOT_SPEED;
                }
                movement=1;
            }
        }
        // is there any alien ship?
        if (num_aliens==0) {
            num_aliens++;
            alien_x=WIDTH-1;
            alien_y=random(0,HEIGHT);
            screen[alien_x-1][alien_y]=ALIEN_SHIP_LX;
            screen[alien_x][alien_y]=ALIEN_SHIP_DX;
            alien_time=millis()+aliens_speed;
            movement=1;
        }
        //alien movement
        if (num_aliens!=0 && millis()>alien_time) {
            death=move_aliens();
            if (death==1) { break; }
            alien_time=millis()+aliens_speed;
            movement=1;
        }
        //does the alien ship fire a shot?
        if (alien_shots==0 && num_aliens!=0) {
            if (millis()>next_alien_shot ) {
                alien_shots++;
                alien_shot_x=alien_x-2;
                alien_shot_y=alien_y;
                alien_shot_time=millis()+SHOT_SPEED;
                movement=1;
            }
        }            
        //alien shot
        if (alien_shots != 0 ) {
            if (millis()>alien_shot_time) {
                death=move_alien_shots();
                if (death==1) { break; }
                if (alien_shots!=0) { 
                    alien_shot_time=millis()+SHOT_SPEED;
                }
                if (alien_shots == 0 ) {
                    next_alien_shot=millis()+5000;
                    if (level < 10 ) { next_alien_shot -= (level*250); }
                }
                movement=1;
            }
        }
        if (movement!=0) {
            //the player has to be moved
            screen[player_x][player_y]=BLANK;
            if (screen[x][y]==ALIEN_SHIP_LX || screen[x][y]==ALIEN_SHIP_DX || screen[x][y]==PILL) {
                //player died
                make_explosion(player_x, player_y, 0);
                death=1;
                break;
            }
            screen[x][y]=PLAYER_SHIP;
            player_x=x;
            player_y=y;
            draw_display();
        }            /*
            //alien shot movement
            time=millis();
            //calculate if the alien shot has to be moved
            if (time-prev_time>ALIEN_SHOT_SPEED) {
                prev_time=millis();
                death=alien_shot_movement();
                if (death!=0) { break; }
                draw_display();
            }*/
    } while (exit_game==0 && death==0);
    lcd.LCD_3310_clear();
    //game over
    game_over(score);
}


//explosion animation
void make_explosion(byte pos_x, byte pos_y, byte two_expl) {
byte j;
    
    for(j=0;j<4;j++) {
        printat(pos_x,pos_y,EXPLOSION);
        if (two_expl != 0) {
            printat(pos_x-1,pos_y,EXPLOSION);
        }
        delay(50);
        printat(pos_x,pos_y,BLANK);
        if (two_expl != 0) {
            printat(pos_x-1,pos_y,BLANK);
        }
        delay(50);
    }
}    


//Move alien ships
byte move_aliens() {
int i,j;
int x1,y1;

    x1=alien_x;
    y1=alien_y;
    i=random(0,100);
    if (i<10) { //randomly the alien ship moves up or down
        j=random(0,2)-1;
        if ((y1+j)>=0 && (y1+j)<HEIGHT) {
            y1+=j;
        }
    }
    x1--;
    if (x1<0) { 
        //alien ship exited from left side
        num_aliens--;
        screen[alien_x][alien_y]=BLANK;
        return 0;
    }
    j=0;
    if (x1>=0) {
       if (screen[x1-1][y1]==PLAYER_SHIP || screen[x1][y1]==PLAYER_SHIP) { 
           j=1;
       }
    } else {
        if (screen[x1][y1]==PLAYER_SHIP) { 
            j=1; 
        }
    }
    if (j==1) {
        make_explosion(player_x, player_y, 0);
        return 1;
    }
    if (alien_x>0) { 
        screen[alien_x-1][alien_y]=BLANK;
        screen[alien_x][alien_y]=BLANK;
    } else if (alien_x==0) {
        screen[alien_x][alien_y]=BLANK;
    } else {
        num_aliens--;
        //screen[alien_x][alien_y]=BLANK;
        return 0;
    }
    alien_x=x1;
    alien_y=y1;
    if ( alien_x > 0 ) {
        screen[alien_x-1][alien_y]=ALIEN_SHIP_LX;
        screen[alien_x][alien_y]=ALIEN_SHIP_DX;
    } else if (alien_x == 0) {
        screen[alien_x][alien_y]=ALIEN_SHIP_DX;
    }
    return 0;
}


//Move player shots
void move_player_shots() {
byte i,j;

    if (shot_x+1 >= WIDTH) {
        screen[shot_x][shot_y]=BLANK;
        player_shots--;
        return;
    }
    i=screen[shot_x+1][shot_y];
    if (i==BLANK) {
        screen[shot_x][shot_y]=BLANK;
        shot_x++;
        screen[shot_x][shot_y]=PLAYER_SHOT;
    }
    if (i==ALIEN_SHIP_LX || i==ALIEN_SHIP_DX) {
        score++;
        screen[shot_x][shot_y]=BLANK;
        screen[alien_x-1][alien_y]=BLANK;
        screen[alien_x][alien_y]=BLANK;
        make_explosion(alien_x, alien_y, 1);
        player_shots--;
        num_aliens--;
        return;
    }
}


//Move alien shots
byte move_alien_shots() {
int i,j;

    if (alien_shot_x-1 < 0) {
        screen[alien_shot_x][alien_shot_y]=BLANK;
        alien_shots--;
        return 0;
    }
    i=screen[alien_shot_x-1][alien_shot_y];
    if (i==BLANK) {
        screen[alien_shot_x][alien_shot_y]=BLANK;
        alien_shot_x--;
        screen[alien_shot_x][alien_shot_y]=PILL;
        return 0;
    } 
    if (i==PLAYER_SHOT) {
        screen[alien_shot_x][alien_shot_y]=BLANK;
        screen[shot_x][shot_y]=BLANK;
        player_shots--;
        alien_shots--;
        return 0;
    }
    if (i==PLAYER_SHIP) {
        make_explosion(player_x,player_y,0);
        return 1;
    }
}


//increment Space-Ace score
void spaceace_score(int addscore) {
    score+= addscore;
    if (score%10==0) {
        spaceace_level_up();
    }
}


//upgrade Space-Ace level
void spaceace_level_up() {
    level++;
    aliens_speed-=50;
    if (aliens_speed < MAX_ALIEN_SPEED) { 
        aliens_speed=MAX_ALIEN_SPEED; 
    }
}


//Initialize Space-Ace game field
void spaceace_init_display() {
byte i,j;

    for (i=0;i<WIDTH;i++) {
        for (j=0;j<HEIGHT;j++) {
            screen[i][j]=BLANK;
        }
    }
    player_x=0;
    player_y=2;
    screen[player_x][player_y]=PLAYER_SHIP;
}


// show the initial splash with the hints of the games
byte hints(char* game_hints[], byte pos_hints[], byte num_hints ) {
    long prev_time;
    unsigned long time;
    char user_exit, done;
    byte i;
    
    //menu initialization
    lcd.LCD_3310_init();
    lcd.LCD_3310_clear();   
    prev_time=millis();
    user_exit=0;
    done=0;
    for(i=0;i<num_hints;i++) {
        lcd.LCD_3310_write_string(pos_hints[i],i,game_hints[i],MENU_NORMAL);
    }
    lcd.LCD_3310_write_string(27,5,"PLAY!",MENU_HIGHLIGHT);
    for(i=0;i<NUM_KEYS;i++) {
        button_flag[i]=0;
    }
    do {
        time=millis();
        if (time-prev_time>10000 && done==0) {
            lcd.LCD_3310_light(LIGHT_OFF); //light off the display after 10 sec.
            done=1;
        }
        for(i=0;i<NUM_KEYS;i++) {
            if(button_flag[i] !=0){
                button_flag[i]=0;  // reset button flag
                switch(i){
                    case CENTER_KEY:
                        user_exit=1;
                        break;
                    case RIGHT_KEY:
                        user_exit=1;
                        break;
                    case LEFT_KEY:
                        user_exit=2;
                        break;
                }
            }
        }
    } while (user_exit==0);
    lcd.LCD_3310_light(LIGHT_ON);
    return user_exit;
}


/* menu functions */

// waiting for center key press
void waitfor_OKkey(){
    byte i;
    byte key = 0xFF;

    while (key!= CENTER_KEY){
        for(i=0; i<NUM_KEYS; i++){
            if(button_flag[i] !=0){
                button_flag[i]=0;  //reset button flag
                if(i== CENTER_KEY) key=CENTER_KEY;
            }
        }
    }		
}


//print a character at x,y coords
void printat(char x, char y, char character) {
    lcd.LCD_3310_set_XY(x*6,y);
    lcd.LCD_3310_write_char(character, MENU_NORMAL);
}


//************************************************************************
//************************************************************************
//************************************************************************
// The followinging are interrupt-driven keypad reading functions
//  which includes DEBOUNCE ON/OFF mechanism, and continuous pressing detection


// Convert ADC value to key number
char get_key(unsigned int input)
{
	char k;
    
	for (k = 0; k < NUM_KEYS; k++)
	{
		if (input < adc_key_val[k])
		{
           
    return k;
        }
	}
    
    if (k >= NUM_KEYS)
        k = -1;     // No valid key pressed
    
    return k;
}

void update_adc_key(){
  int adc_key_in;
  char key_in;
  byte i;
  
  adc_key_in = analogRead(0);
  key_in = get_key(adc_key_in);
  for(i=0; i<NUM_KEYS; i++)
  {
    if(key_in==i)  //one key is pressed 
    { 
      if(button_count[i]<DEBOUNCE_MAX)
      {
        button_count[i]++;
        if(button_count[i]>DEBOUNCE_ON)
        {
          if(button_status[i] == 0)
          {
            button_flag[i] = 1;
            button_status[i] = 1; //button debounced to 'pressed' status
          }
		  
        }
      }
	
    }
    else // no button pressed
    {
      if (button_count[i] >0)
      {  
		button_flag[i] = 0;	
		button_count[i]--;
        if(button_count[i]<DEBOUNCE_OFF){
          button_status[i]=0;   //button debounced to 'released' status
        }
      }
    }
    
  }
}

// Timer2 interrupt routine -
// 1/(160000000/256/(256-6)) = 4ms interval

ISR(TIMER2_OVF_vect) {  
  TCNT2  = 6;
  update_adc_key();
}

